{{ port.name }} provides CMake targets:

    find_package({{ port.name }} CONFIG REQUIRED)
    target_link_libraries(main PRIVATE {% if port.target_override %}{{ port.target_override }}{% else %}portal::{{ port.short_name }}{% endif %})